*===========================================================================*
* Program.........:HlpHand                                                  *
* Author..........:Gordon Walker                                            *
* Project.........:LandMark for Windows                                     *
* Description.....:Handler for context sensitive help. It should be hooked  *
*                 :to F1 via an ON KEY LABEL F1 and care should be taken to *
*                 :see that this OKL is not cleared. The OKL creation and   *
*                 :other initialisation items are handled by a call to the  *
*                 :Intiialise procedure defined within. It should be called *
*                 :before any help operation is attempted. HlpHand makes    *
*                 :use of a database called HELPMAP; this contains a number *
*                 :of unique identifiers composed of a global setting help  *
*                 :in gcMenuOption, the current WONTOP and the current      *
*                 :VARREAD. For use, each menu option should call the       *
*                 :functions MenuSel and MenuNoSel before and after their   *
*                 :main operation to set gcMenuOption. Alternatively, if    *
*                 :the command to be accomplished by the menu option is a   *
*                 :simple DO, this command can be passed, along with the    *
*                 :selection id, to the alternative routine MenuCall which  *
*                 :incorporates the functionality of MenuSel and MenuNoSel  *
*                 :and also executes the DO. The level of detail can be     *
*                 :varied throughout the application, this handler will     *
*                 :search first for an entry in HELPMAP for the current     *
*                 :READ object in the WONTOP for gcMenuOption. If there is  *
*                 :no match it searchs for a general entry for this WONTOP  *
*                 :in gcMenuOption. If there is no match, it searches for a *
*                 :generic window entry (ie. a window which occurs under    *
*                 :several menu options like the login screen) signified by *
*                 :the window name preceded with an asterix. If there is no *
*                 :match it searches for a general entry for gcMenuOption.  *
*                 :If there is no match it displays the main help index.    *
* Pre.............:The databases HELP and HELPMAP are available.            *
*                 :gcMenuOption has been declared PUBLIC.                   *
* Post............:None                                                     *
* Calling Sample..:ON KEY LABEL F1 DO HlpHand                               *
* Parameter List..:None                                                     *
*---------------------------------------------------------------------------*
*                             Major Change List                             *
*                                                                           *
*===========================================================================*
* This will be expanded later to allow switching between DOS and Windows 
* style help systems.
#DEFINE HELPSTYLE "DOS"
PRIVATE m.luCurrArea

* Clear any OKLs
PUSH KEY CLEAR

* Store the current work area
luCurrArea=SELECT(0)

* Fix the setting of SET EXACT
m.lcExact=SET("EXACT")
SET EXACT ON

IF NOT(USED("helpmap")
	SELECT 0
	USE helpmap
ELSE
	SELECT helpmap
ENDIF
SET ORDER TO id

* Seek first for help on the current variable
IF !SEEK(ALLTRIM(UPPER(gcMenuOption))+ALLTRIM(UPPER(WONTOP()))+ALLTRIM(VARREAD()))
	* Seek for help on the current window
	IF !SEEK(ALLTRIM(UPPER(gcMenuOption))+ALLTRIM(UPPER(WONTOP())))
		* Seek for help on a generic window
		IF !SEEK("*"+ALLTRIM(UPPER(WONTOP())))
			* Seek for help on the current menu option
			IF EMPTY(gcMenuOption) OR !SEEK(ALLTRIM(UPPER(gcMenuOption)))
				* No specific help available, the first entry in HELPMAP
				* is null.
				GO TOP
			ENDIF
		ENDIF
	ENDIF
ENDIF

* Call the appropriate help
IF HELPSTYLE="DOS"
	IF EMPTY(HelpMap.Topic)
		SET TOPIC TO
	ELSE
		SET TOPIC TO HelpMap.Topic
	ENDIF
	HELP
ENDIF

* Restore the setting of SET EXACT
IF lcExact="OFF"
	SET EXACT OFF
ENDIF

* Restore work area
SELECT (luCurrArea)

* Restore OKL definitions
POP KEY

*---------------------------------------------------------------------------*
* Procedure.......:MenuSel                                                  *
* Author..........:Gordon Walker                                            *
* Description.....:Called before every menu option to set the global        *
*                 :variable gcMenuOption to represent the current menu      *
*                 :selected process.                                        *
* Calling Sample..:DO MenuSel IN HlpHand WITH "PREPARE"                     *
* Parameter List..:m.tcSelection - the unique and predetermined identifier  *
*                 :for the current menu selected process.                   *
*---------------------------------------------------------------------------*
*                             Major Change List                             *
*                                                                           *
*---------------------------------------------------------------------------*
PROCEDURE MenuSel
PARAMETERS m.tcSelection
* Set the global variable
m.gcMenuOption=UPPER(m.tcSelection)

*---------------------------------------------------------------------------*
* Procedure.......:MenuNoSel                                                *
* Author..........:Gordon Walker                                            *
* Description.....:Called after a menu operation has completed, this clears *
*                 :the global variable gcMenuOption to indicate to the      *
*                 :context sensitive help apparatus that it should display  *
*                 :general help.                                            *
* Calling Sample..:DO MenuNoSel IN HlpHand                                  *
* Parameter List..:None                                                     *
*---------------------------------------------------------------------------*
*                             Major Change List                             *
*                                                                           *
*---------------------------------------------------------------------------*
PROCEDURE MenuNoSel
m.gcMenuOption=""

*---------------------------------------------------------------------------*
* Procedure.......:MenuCall                                                 *
* Author..........:Gordon Walker                                            *
* Description.....:Combines the functionality of MenuSel and MenuNoSel.     *
*                 :Useful if the menu option has only to execute a DO       *
*                 :command. This command is passed to the procedure along   *
*                 :with the selection id. The global variable is set, the   *
*                 :command executed and the global variable cleared in a    *
*                 :single routine. In addition, a call is made to ChkRes to *
*                 :determine if there are sufficent resources before        *
*                 :executing the passed command                             *
* Pre.............:                                                         *
* Post............:                                                         *
* Calling Sample..:DO MenuCall WITH "PREPARE.PRG","PREPARE"                 *
* Parameter List..:tcCommand - a command capable of execution via DO        *
*                 :&tcCommand, which accomplishes the requirements of the   *
*                 :menu option tcSelection - appropriate id for use in the  *
*                 :HelpMap                                                  *
*---------------------------------------------------------------------------*
*                             Major Change List                             *
*                                                                           *
*---------------------------------------------------------------------------*
PROCEDURE MenuCall
PARAMETERS m.tcCommand, m.tcSelection
* Establish the current selection
DO MenuSel WITH tcSelection
DO &tcCommand
* Clear the selection
DO MenuNoSel

*---------------------------------------------------------------------------*
* Procedure.......:Initialise                                               *
* Author..........:Gordon Walker                                            *
* Description.....:A procedure to be called to prepare for context          *
*                 :sensitive help operation. It creates the required public *
*                 :variable gcMenuOption, set the help database to be       *
*                 :HELP.DBF and hooks F1 to call the help handler           *
* Pre.............:HELP.DBF and HELPMAP.DBF must be available.              *
* Post............:The public variable gcMenuOption has been created and    *
*                 :initialised as null. Key F1 has been  ON KEY LABELLED to *
*                 :call HlpHand. The current help database is HELP.DBF and  *
*                 :help is active.                                          *
* Calling Sample..:DO Initialise IN HlpHand                                 *
* Parameter List..:None                                                     *
*---------------------------------------------------------------------------*
*                             Major Change List                             *
*                                                                           *
*---------------------------------------------------------------------------*
PROCEDURE Initialise
* Create the public variable which is to hold the current menu selection
PUBLIC m.gcMenuOption
gcMenuOption=""
* Hook in the help system
ON KEY LABEL F1 DO HlpHand
SET HELP TO help.dbf
SET HELP ON

